////////////////////////////////////////////////////////////////////////////////////////////
// SX18/20/28/48/52 universal config utility
////////////////////////////////////////////////////////////////////////////////////////////
// Author(s): 	Marc Gebauer
//				Chris Fogelklou
//				ANAND.K.C
// Date 31 May 2005
////////////////////////////////////////////////////////////////////////////////////////////

//*********************************************************************************
// copy following function into main()
//*********************************************************************************
/*
	sx_config()
*/
//*********************************************************************************
//	 RTCC interrupt, WREG register, RTCC/WDT prescaler definitions
//*********************************************************************************

#pragma TURBO_MODE 1
#pragma CLOCK_FREQ 50000000

//boolean equates
#define TRUE 1
#define FALSE 0

#ifdef _SX48BD
#define SX48_52
#endif

#ifdef _SX52BD
#define SX48_52
#endif

#ifdef SX48_52

/*****************************************************************************************
  SX48BD/52BD Mode addresses
  *On SX48BD/52BD, most registers addressed via mode are read and write, with the
  exception of CMP and WKPND which do an exchange with W.
 *****************************************************************************************/
//---------------------------------- Timer (read) addresses--------------------------------

#define TCPL_R		0x00		//Read Timer Capture register low byte
#define TCPH_R		0x01		//Read Timer Capture register high byte
#define TR2CML_R	0x02		//Read Timer R2 low byte
#define TR2CMH_R	0x03		//Read Timer R2 high byte
#define TR1CML_R	0x04		//Read Timer R1 low byte
#define TR1CMH_R	0x05 		//Read Timer R1 high byte
#define TCNTB_R		0x06		//Read Timer control register B
#define TCNTA_R		0x07		//Read Timer control register A

//--------------------------------- Exchange addresses------------------------------------

#define CMP			0x08		//Exchange Comparator enable/status register with W
#define WKPND		0x09		//Exchange MIWU/RB Interrupts pending with W

//---------------------------------port setup (read) addresses----------------------------

#define WKED_R		0x0A		//Read MIWU/RB Interrupt edge setup, 1 = falling, 0 = rising
#define WKEN_R		0x0B		//Read MIWU/RB Interrupt edge setup, 0 = enabled, 1 = disabled
#define ST_R		0x0C		//Read Port Schmitt Trigger setup, 0 = enabled, 1 = disabled
#define LVL_R		0x0D		//Read Port Schmitt Trigger setup, 0 = enabled, 1 = disabled
#define PLP_R		0x0E		//Read Port Schmitt Trigger setup, 0 = enabled, 1 = disabled
#define DDIR_R		0x0F		//Read Port Direction

//----------------------------------Timer (write) addresses-------------------------------

#define CLR_TMR		0x10		//Resets 16-bit Timer
#define TR2CML_W	0x12		//Write Timer R2 low byte
#define TR2CMH_W	0x13		//Write Timer R2 high byte
#define TR1CML_W	0x14		//Write Timer R1 low byte
#define TR1CMH_W	0x15 		//Write Timer R1 high byte
#define TCNTB_W		0x16		//Write Timer control register B
#define TCNTA_W		0x17		//Write Timer control register A

//------------------------------------Port setup (write) addresses------------------------

#define WKED_W		0x1A		//Write MIWU/RB Interrupt edge setup, 1 = falling, 0 = rising
#define WKEN_W		0x1B		//Write MIWU/RB Interrupt edge setup, 0 = enabled, 1 = disabled
#define ST_W		0x1C		//Write Port Schmitt Trigger setup, 0 = enabled, 1 = disabled
#define LVL_W		0x1D		//Write Port Schmitt Trigger setup, 0 = enabled, 1 = disabled
#define PLP_W		0x1E		//Write Port Schmitt Trigger setup, 0 = enabled, 1 = disabled
#define DDIR_W		0x1F		//Write Port Direction

#else

/*****************************************************************************************
  SX18AC/20AC/28AC Mode addresses
  *On SX18/20/28, all registers addressed via mode are write only, with the exception of
  CMP and WKPND which do an exchange with W.
 *****************************************************************************************/

//-----------------------------------------Exchange addresses-----------------------------

#define CMP			0x08		//Exchange Comparator enable/status register with W
#define WKPND		0x09		//Exchange MIWU/RB Interrupts pending with W

//----------------------------------------Port setup (read) addresses---------------------

#define WKED_W		0x0A		//Write MIWU/RB Interrupt edge setup, 1 = falling, 0 = rising
#define WKEN_W		0x0B		//Write MIWU/RB Interrupt edge setup, 0 = enabled, 1 = disabled
#define ST_W		0x0C		//Write Port Schmitt Trigger setup, 0 = enabled, 1 = disabled
#define LVL_W		0x0D		//Write Port Schmitt Trigger setup, 0 = enabled, 1 = disabled
#define PLP_W		0x0E		//Write Port Schmitt Trigger setup, 0 = enabled, 1 = disabled
#define DDIR_W		0x0F		//Write Port Direction

#endif

/*****************************************************************************************
 ---------------------------------- Port Assignment---------------------------------------
 *****************************************************************************************/

#define RA_latch	00001000b		//SX18/20/28/48/52 port A latch init
#define RA_DDIR		11110111b		//SX18/20/28/48/52 port A DDIR value
#define RA_LVL		00000000b		//SX18/20/28/48/52 port A LVL value
#define RA_PLP		00001100b		//SX18/20/28/48/52 port A PLP value

#define RB_latch	11111111b		//SX18/20/28/48/52 port B latch init;intial value after reset
#define RB_DDIR		11111111b		//SX18/20/28/48/52 port B DDIR value; 0=Output,1=Input
#define RB_ST		00000000b		//SX18/20/28/48/52 port B ST value; 0=Enable,1=Disable
#define RB_LVL		00000000b		//SX18/20/28/48/52 port B LVL value; 0=CMOS,1=TTL
#define RB_PLP		00000000b		//SX18/20/28/48/52 port B PLP value; 0=Enable,1=Disable

#define RC_latch	10000000b		//SX18/20/28/48/52 port C latch init;intial value after reset
#define RC_DDIR		01111111b		//SX18/20/28/48/52 port C DDIR value; 0=Output,1=Input
#define RC_ST		11111111b		//SX18/20/28/48/52 port C ST value; 0=Enable,1=Disable
#define RC_LVL		00000000b		//SX18/20/28/48/52 port C LVL value; 0=CMOS,1=TTL
#define RC_PLP		10000000b		//SX18/20/28/48/52 port C PLP value; 0=Enable,1=Disable

#ifdef SX48_52		

#define RD_latch	00000000b		//SX48/52 port D latch init;intial value after reset
#define RD_DDIR		11111111b		//SX48/52 port D DDIR value; 0=Output,1=Input
#define RD_ST		11111111b		//SX48/52 port D ST value; 0=Enable,1=Disable
#define RD_LVL		00000000b		//SX48/52 port D LVL value; 0=CMOS,1=TTL
#define RD_PLP		00000000b		//SX48/52 port D PLP value; 0=Enable,1=Disable

#define RE_latch	00000000b		//SX48/52 port E latch init;intial value after reset
#define RE_DDIR		11111111b		//SX48/52 port E DDIR value; 0=Output,1=Input
#define RE_ST		11111111b		//SX48/52 port E ST value; 0=Enable,1=Disable
#define RE_LVL		00000000b		//SX48/52 port E LVL value; 0=CMOS,1=TTL
#define RE_PLP		00000000b		//SX48/52 port E PLP value; 0=Enable,1=Disable

#endif

/*****************************************************************************************
 ---------------------------- Initialise all port configuration --------------------------
 *****************************************************************************************/

void sx_config(void)				// call this routine in void main(void)
{
		set_mode(ST_W);				//point MODE to write ST register
		asm mov	w,#RB_ST			;Setup RB Schmitt Trigger, 0 = enabled, 1 = disabled
		asm mov	!rb,w		
		asm mov	w,#RC_ST			;Setup RC Schmitt Trigger, 0 = enabled, 1 = disabled
		asm mov	!rc,w	
#ifdef SX48_52
		asm mov	w,#RD_ST			;Setup RD Schmitt Trigger, 0 = enabled, 1 = disabled
		asm mov	!rd,w		
		asm mov	w,#RE_ST			;Setup RE Schmitt Trigger, 0 = enabled, 1 = disabled
		asm mov	!re,w		
#endif
		set_mode(LVL_W);			//point MODE to write LVL register
		asm mov	w,#RA_LVL			;Setup RA CMOS or TTL levels, 1 = TTL, 0 = CMOS
		asm mov	!ra,w		 
		asm mov	w,#RB_LVL			;Setup RB CMOS or TTL levels, 1 = TTL, 0 = CMOS,0,1 = TTL, 2..7 = CMOS
		asm mov	!rb,w		
		asm mov	w,#RC_LVL			;Setup RC CMOS or TTL levels, 1 = TTL, 0 = CMOS
		asm mov	!rc,w	
#ifdef SX48_52
		asm mov	w,#RD_LVL			;Setup RD CMOS or TTL levels, 1 = TTL, 0 = CMOS
		asm mov	!rd,w		
		asm mov	w,#RE_LVL			;Setup RE CMOS or TTL levels, 1 = TTL, 0 = CMOS
		asm mov	!re,w		
#endif
		set_mode(PLP_W);			//point MODE to write PLP register
		asm mov	w,#RA_PLP			;Setup RA Weak Pull-up, 0 = enabled, 1 = disabled
		asm mov	!ra,w		 
		asm mov	w,#RB_PLP			;Setup RB Weak Pull-up, 0 = enabled, 1 = disabled
		asm mov	!rb,w		
		asm mov	w,#RC_PLP			;Setup RC Weak Pull-up, 0 = enabled, 1 = disabled
		asm mov	!rc,w	
#ifdef SX48_52
		asm mov	w,#RD_PLP			;Setup RD Weak Pull-up, 0 = enabled, 1 = disabled
		asm mov	!rd,w		
		asm mov	w,#RE_PLP			;Setup RE Weak Pull-up, 0 = enabled, 1 = disabled
		asm mov	!re,w		
#endif
		set_mode(DDIR_W);			//point MODE to write DDIR register
		asm mov	w,#RA_DDIR			;Setup RA Direction register, 0 = output, 1 = input		
		asm mov	!ra,w	
		asm mov	w,#RB_DDIR			;Setup RB Direction register, 0 = output, 1 = input
		asm mov	!rb,w			
		asm mov	w,#RC_DDIR			;Setup RC Direction register, 0 = output, 1 = input
		asm mov	!rc,w			
#ifdef SX48_52
		asm mov	w,#RD_DDIR			;Setup RD Direction register, 0 = output, 1 = input
		asm mov	!rd,w			
		asm mov	w,#RE_DDIR			;Setup RE Direction register, 0 = output, 1 = input
		asm mov	!re,w			
#endif
		asm mov	w,#RA_latch			;Initialize RA data latch
		asm mov	ra,w		
		asm mov	w,#RB_latch			;Initialize RB data latch
		asm mov	rb,w		
		asm mov	w,#RC_latch			;Initialize RC data latch
		asm mov	rc,w		
#ifdef SX48_52
		asm mov	w,#RD_latch			;Initialize RD data latch
		asm mov	rd,w			
		asm mov	w,#RE_latch			;Initialize RE data latch
		asm mov	re,w			
#endif
    
/*********************************************************************************
	 Setup and enable RTCC interrupt, WREG register, RTCC/WDT prescaler
*********************************************************************************/

#define RTCC_ON			0x0080		//Enables RTCC at address $01 (RTW hi)
									//*WREG at address $01 (RTW lo) by default
#define RTCC_ID			0x0040		//Disables RTCC edge interrupt (RTE_IE hi)
									//*RTCC edge interrupt (RTE_IE lo) enabled by default
#define RTCC_INC_EXT	0x0020		//Sets RTCC increment on RTCC pin transition (RTS hi)
									//*RTCC increment on internal instruction (RTS lo) is defalut
#define RTCC_FE			0x0010		//Sets RTCC to increment on falling edge (RTE_ES hi)
									//*RTCC to increment on rising edge (RTE_ES lo) is default
#define RTCC_PS_ON		0x0000		//Assigns prescaler to RTCC (PSA lo)
#define RTCC_PS_OFF		0x0008		//Assigns prescaler to RTCC (PSA lo)
#define PS_000			0x0000		//RTCC = 1:2, WDT = 1:1
#define PS_001			0x0001		//RTCC = 1:4, WDT = 1:2
#define PS_010			0x0002		//RTCC = 1:8, WDT = 1:4
#define PS_011			0x0003		//RTCC = 1:16, WDT = 1:8
#define PS_100			0x0004		//RTCC = 1:32, WDT = 1:16
#define PS_101			0x0005		//RTCC = 1:64, WDT = 1:32
#define PS_110			0x0006		//RTCC = 1:128, WDT = 1:64
#define PS_111			0x0007		//RTCC = 1:256, WDT = 1:128

    set_option( RTCC_ON|RTCC_PS_OFF ); //set divider 1:256
}
